Package: bcp
Type: Package
Title: Bayesian Analysis of Change Point Problems
Version: 4.0.4
Date: 2026-05-12
Authors@R: c(person(given = "Xiaofei",
                        family = "Wang",
                        role =  "aut" ),
                 person(given = "Chandra",
                        family = "Erdman",
                        role = "aut"),
                 person(given = c("John", "W."),
                        family = "Emerson",
                        role = "aut"),
                 person(given = "Kaiguang",
                        family = "Zhao",
                        role = c("aut", "cre"),
                        email = "zhao.1423@osu.edu")
 			)
Depends: graphics, methods, grid
Suggests: DNAcopy, coda, strucchange, vegan, ggplot2, igraph
Description: Provides an implementation of the product partition model described in Barry and Hartigan (2019) <doi:10.2307/2290726> for the normal errors change point problem using Markov Chain Monte Carlo (MCMC). It also extends the methodology to regression models on a connected graph as reported in Wang and Emerson (2015) <doi:10.48550/arXiv.1509.00817>, allowing estimation of change point models with multivariate responses. Parallel MCMC, previously available in 'bcp' v.3.0.0, is currently not implemented.
License: GPL (>= 2)
Imports: Rcpp (>= 0.11.0)
LinkingTo: Rcpp, RcppArmadillo
LazyData: true
LazyLoad: true
NeedsCompilation: yes
URL: https://github.com/zhaokg/bcp
BugReports: https://github.com/zhaokg/bcp/issues
Repository: CRAN
Encoding: UTF-8
Packaged: 2026-05-16 02:23:03 UTC; zhaokg
Author: Xiaofei Wang [aut],
  Chandra Erdman [aut],
  John W. Emerson [aut],
  Kaiguang Zhao [aut, cre]
Maintainer: Kaiguang Zhao <zhao.1423@osu.edu>
Date/Publication: 2026-05-20 08:50:10 UTC
