# bcp
`bcp` is an R package that provides an implementation of the Barry and Hartigan (1993) product partition model for the normal errors change point problem using Markov Chain Monte Carlo.  It also extends the methodology to regression models on a connected graph (Wang and Emerson, 2015); this allows estimation of change point models with multivariate responses. Parallel MCMC, previously available in bcp v.3.0.0, is currently not implemented.

To install:

* the latest released version: `install.packages("bcp")`
* the latest development version: `install_github("zhaokg/bcp")`
